/* $Id: Bucket-test.cpp,v 1.5 2003/03/11 22:37:38 lergnom Exp $ -*- C++ -*-
 * 
 * libbpfl - Bayesian Probability Filtering Library
 * Copyright (C) 2003 Anthony Ventimiglia
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
 */
/**
 * @file   Bucket-test.cpp
 * @author Anthony Ventimiglia
 * @date   Mon Mar 10 15:42:15 2003
 * 
 * @brief  This is a trest of the bpf::Bucket class.
 * 
 * 
 */

#include "bpfBucket.h"
#include "bpfHash.h"

#include <string>
#include <iostream>
#include <map>

char Bucket_File[] = "Bucket-test.dat";

const int LSize = 181; 
// This is the order that the Hash_map should end up in.
// It may change if the internals, mainly the resize policy changes
char* wordList[] = { 

    // 17 --------- RISC (/ 17 LS)  0.09392265193370165
    // 18 ---- abasement (/ 18 LS)  0.09944751381215469
    // 8 ----------- beer (/ 8 LS)  0.04419889502762431
    // 7 ------- chancery (/ 7 LS)  0.03867403314917127
    // 6 ------- decently (/ 6 LS)  0.03314917127071823
    // 5 ------- emulator (/ 5 LS)  0.027624309392265192
    // 4 ------- firewood (/ 4 LS)  0.022099447513812154
    // 3 ----------- gram (/ 3 LS)  0.016574585635359115
    // 21 -------- heaver (/ 2 LS)  0.011049723756906077
    // 1 ----- infernally (/ 1 LS)  0.0055248618784530384
    // 10 -------- junta (/ 10 LS)  0.055248618784530384
    // 11 ------ kinsman (/ 11 LS)  0.06077348066298342
    // 12 - laboratories (/ 12 LS)  0.06629834254143646
    // 13 ---- marijuana (/ 13 LS)  0.0718232044198895
    // 14 ---- neutrinos (/ 14 LS)  0.07734806629834254
    // 15 ------ orchids (/ 15 LS)  0.08287292817679558
    // 16 ----- patties  (/ 16 LS)  0.08839779005524862
    "RISC", "RISC", "RISC", "RISC", "RISC", "RISC", "RISC", "RISC",  
    "RISC", "RISC", "RISC", "RISC", "RISC", "RISC", "RISC", "RISC", 
    "RISC", "RISC",
    "abasement", "abasement", "abasement", "abasement", "abasement", 
    "abasement", "abasement", "abasement", "abasement", "abasement", 
    "abasement", "abasement", "abasement", "abasement", "abasement", 
    "abasement", "abasement", "abasement",
    "beer", "beer", "beer", "beer", "beer", "beer", "beer", "beer", 
    "chancery", "chancery", "chancery", "chancery", "chancery", "chancery", 
    "chancery", 
    "decently", "decently", "decently", "decently", "decently", "decently", 
    "emulator", "emulator", "emulator", "emulator", "emulator", 
    "firewood", "firewood", "firewood", "firewood", 
    "gram", "gram", "gram",
    "heaver", "heaver", "heaver", "heaver", "heaver", "heaver",
    "heaver", "heaver", "heaver", "heaver", "heaver", "heaver",
    "heaver", "heaver", "heaver", "heaver", "heaver", "heaver",
    "heaver", "heaver", "heaver",
    "infernally",
    "junta", "junta", "junta", "junta", "junta", "junta", "junta",
    "junta", "junta", "junta",
    "kinsman", "kinsman", "kinsman", "kinsman", "kinsman", "kinsman",
    "kinsman", "kinsman", "kinsman", "kinsman", "kinsman",
    "laboratories", "laboratories", "laboratories", "laboratories",
    "laboratories", "laboratories", "laboratories", "laboratories",
    "laboratories", "laboratories", "laboratories", "laboratories",
     "marijuana", "marijuana", "marijuana", "marijuana", "marijuana",
    "marijuana", "marijuana", "marijuana", "marijuana", "marijuana",
    "marijuana", "marijuana", "marijuana",
    "neutrinos", "neutrinos", "neutrinos", "neutrinos", "neutrinos", 
    "neutrinos", "neutrinos", "neutrinos", "neutrinos", "neutrinos",
    "neutrinos", "neutrinos", "neutrinos", "neutrinos", 
    "orchids", "orchids", "orchids", "orchids", "orchids", "orchids",
    "orchids", "orchids", "orchids", "orchids", "orchids", "orchids", 
    "orchids", "orchids", "orchids",
    "patties", "patties", "patties", "patties", "patties", "patties",
    "patties", "patties", "patties", "patties", "patties", "patties",
    "patties", "patties", "patties", "patties",
};


struct Hash : public bpf::Hash_base<std::string>
{
    size_t operator() ( const std::string& t ) 
	{
	    using std::string;
	    size_t hval = 0;
	    for ( string::const_iterator si = t.begin(); si != t.end(); ++si )
		hval = ( hval << 1 ) ^ int (*si);
	    return hval;
	}
};



int main ()
{
    using namespace std;
    
    typedef bpf::Bucket<string, Hash> wBucket;
    wBucket words ( "words" );
    map<string, int> wmap;

    //insert words
    for ( int i = 0; i < LSize; ++i ){
	string word = wordList[i];
	words.add ( word );
	wmap[word] += 1;
    }
    
    //Print scores
    cout << "Original Word Map Scores:\n"
	 << "Hash_map<string>\tmap<string>\tword\n";
    for ( std::map<string, int>::const_iterator mi = wmap.begin(); 
	  mi != wmap.end();
	  ++mi ) 
	cout << words.score ( mi->first ) << "\t==\t" 
	     << double ( mi->second ) / double ( LSize )
	     << '\t' << mi->first << endl;
	

    //Save scores
    words.save ( Bucket_File );
    
    cout << endl;

    //Check Saved File
    wBucket nwb ( "Loaded" );
    nwb.load ( Bucket_File );
    cout << "Saved and Loaded Word Map Scores:\n"
	 << "Hash_map<string>\tmap<string>\tword\n";
    for ( std::map<string, int>::const_iterator mi = wmap.begin(); 
	  mi != wmap.end();
	  ++mi ) 
	cout << nwb.score ( mi->first ) << "\t==\t"
	     << double ( mi->second ) / double ( LSize ) 
	     << '\t' << mi->first << endl;
    


    return 0;
}
